# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from .falai_demo import automask_image as automask_image
from .falai_demo import falai_automask_image as falai_automask_image
from .falai_demo import falai_manuelmask_image as falai_manuelmask_image
from .falai_demo import manuelmask_image as manuelmask_image
from .mask_predictor import SegAutoMaskPredictor as SegAutoMaskPredictor
from .mask_predictor import SegManualMaskPredictor as SegManualMaskPredictor
from .sahi_predict import SahiAutoSegmentation as SahiAutoSegmentation
from .sahi_predict import sahi_sliced_predict as sahi_sliced_predict

__version__ = "0.7.7"



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from .common import LayerNorm2d as LayerNorm2d
from .common import MLPBlock as MLPBlock
from .image_encoder import ImageEncoderViT as ImageEncoderViT
from .mask_decoder import MaskDecoder as MaskDecoder
from .prompt_encoder import PositionEmbeddingRandom as PositionEmbeddingRandom
from .prompt_encoder import PromptEncoder as PromptEncoder
from .sam import Sam as Sam
from .transformer import Attention as Attention
from .transformer import TwoWayAttentionBlock as TwoWayAttentionBlock
from .transformer import TwoWayTransformer as TwoWayTransformer



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from .data_utils import load_box as load_box
from .data_utils import load_image as load_image
from .data_utils import load_mask as load_mask
from .data_utils import load_server_image as load_server_image
from .data_utils import load_video as load_video
from .data_utils import multi_boxes as multi_boxes
from .data_utils import plt_load_box as plt_load_box
from .data_utils import plt_load_mask as plt_load_mask
from .data_utils import read_image as read_image
from .data_utils import save_image as save_image
from .data_utils import show_image as show_image
from .file_utils import download_model as download_model
from .onnx import SamOnnxModel as SamOnnxModel
from .transforms import ResizeLongestSide as ResizeLongestSide



from .automatic_mask_generator import (
    SamAutomaticMaskGenerator as SamAutomaticMaskGenerator,
)
from .build_sam import build_sam as build_sam
from .build_sam import build_sam_vit_b as build_sam_vit_b
from .build_sam import build_sam_vit_h as build_sam_vit_h
from .build_sam import build_sam_vit_l as build_sam_vit_l
from .build_sam import sam_model_registry as sam_model_registry
from .predictor import SamPredictor as SamPredictor



from io import BytesIO

from PIL import Image

from metaseg.mask_predictor import SegAutoMaskPredictor, SegManualMaskPredictor
from metaseg.utils.data_utils import load_server_image

try:
    from fal_serverless import isolated
except ImportError:
    raise ImportError(
        "Please install FalAI library using 'pip install fal_serverless'."
    )


@isolated(requirements=["metaseg"], keep_alive=1800, machine_type="GPU-T4")
def automask_image(
    data, model_type="vit_b", points_per_side=16, points_per_batch=32, min_area=0
):
    image_path, output_path = load_server_image(data)
    SegAutoMaskPredictor().image_predict(
        source=image_path,
        model_type=model_type,
        points_per_side=points_per_side,
        points_per_batch=points_per_batch,
        min_area=min_area,
        output_path=output_path,
        show=False,
        save=True,
    )
    with open(output_path, "rb") as f:
        result = f.read()

    return result


@isolated(requirements=["metaseg"], keep_alive=1800, machine_type="GPU-T4")
def manuelmask_image(
    data,
    model_type="vit_b",
    input_point=[[100, 100], [200, 200]],
    input_label=[0, 1],
    input_box=[100, 100, 200, 200],
    multimask_output=False,
    random_color=False,
    min_area=0,
):
    image_path, output_path = load_server_image(data)
    SegManualMaskPredictor().image_predict(
        source=image_path,
        model_type=model_type,
        input_point=input_point,
        input_label=input_label,
        input_box=input_box,  #
        multimask_output=multimask_output,
        random_color=random_color,
        min_area=min_area,  #
        output_path=output_path,
        show=False,
        save=True,
    )
    with open(output_path, "rb") as f:
        result = f.read()

    return result


def falai_automask_image(
    image_path, model_type="vit_b", points_per_side=16, points_per_batch=32, min_area=0
):
    with open(image_path, "rb") as f:
        data = f.read()

    image = automask_image(
        data=data,
        model_type=model_type,
        points_per_side=points_per_side,
        points_per_batch=points_per_batch,
        min_area=min_area,
    )
    image = Image.open(BytesIO(image))
    return image


def falai_manuelmask_image(
    image_path,
    model_type="vit_b",
    input_point=[[100, 100], [200, 200]],
    input_label=[0, 1],
    input_box=[100, 100, 200, 200],
    multimask_output=False,
    random_color=False,
    min_area=0,
):
    with open(image_path, "rb") as f:
        data = f.read()

    image = manuelmask_image(
        data=data,
        model_type=model_type,
        input_point=input_point,
        input_label=input_label,
        input_box=input_box,
        multimask_output=multimask_output,
        random_color=random_color,
        min_area=min_area,
    )
    image = Image.open(BytesIO(image))
    return image
